using Infrastructure.Options;
using Microsoft.AspNetCore.Authorization;
using Microsoft.IdentityModel.Tokens;

namespace Infrastructure.Extensions;

public static class AuthorizeSetup
{
    /// <summary>
    /// 添加权限认证，权限Policy、Roles从配置中获取
    /// </summary>
    /// <param name="services"></param>
    /// <param name="configuration"></param>
    /// <param name="configureAuthorizeBuilder"></param>
    /// <returns></returns>
    public static IServiceCollection AddDefaultAuthorize(
        this IServiceCollection services,
        IConfiguration configuration,
        Action<AuthorizationBuilder>? configureAuthorizeBuilder = null)
    {
        ArgumentNullException.ThrowIfNull(services);
        ArgumentNullException.ThrowIfNull(configuration);
        var audienceOptions = configuration.GetSection(AudienceOptions.Name).Get<AudienceOptions>();
        if (audienceOptions is null || !audienceOptions.IsEnable)
        {
            return services;
        }

        var key = configuration["AUDIENCE_KEY"] ?? audienceOptions.Secret;
        ArgumentException.ThrowIfNullOrEmpty(key);
        var buffer = Encoding.UTF8.GetBytes(key);
        var securityKey = new SymmetricSecurityKey(buffer);
        var signingCredentials = new SigningCredentials(securityKey, SecurityAlgorithms.HmacSha256);

        services.TryAddSingleton(new JwtContext(
            audienceOptions.Issuer,
            audienceOptions.Audience,
            audienceOptions.Duration,
            signingCredentials));

        var builder = services.AddAuthorizationBuilder();
        builder.AddPolicy(audienceOptions.Policy!, policy =>
            policy.RequireRole(audienceOptions.Roles!)
                .Build());
        configureAuthorizeBuilder?.Invoke(builder);
        return services;
    }
}