using Infrastructure.Filters;
using Infrastructure.Utils;
using Microsoft.AspNetCore.Mvc;
using Newtonsoft.Json;
using Newtonsoft.Json.Converters;
using Newtonsoft.Json.Serialization;

namespace Infrastructure.Extensions;

public static class ControllerSetup
{
    /// <summary>
    /// 配置controller，包含过滤器、json序列化以及模型验证等配置
    /// </summary>
    /// <param name="services"></param>
    /// <param name="configureControllers">配置Controllers</param>
    /// <param name="configureMvcOptions">mvc配置选项</param>
    /// <param name="configureMvcNewtonsoftJsonOptions">json序列化配置选项</param>
    /// <param name="configureApiBehaviorOptions">api行为配置选项</param>
    /// <returns></returns>
    public static IServiceCollection AddDefaultControllers(this IServiceCollection services,
        Action<IMvcBuilder>? configureControllers = null,
        Action<MvcOptions>? configureMvcOptions = null,
        Action<MvcNewtonsoftJsonOptions>? configureMvcNewtonsoftJsonOptions = null,
        Action<ApiBehaviorOptions>? configureApiBehaviorOptions = null)
    {
        ArgumentNullException.ThrowIfNull(services);
        var mvcBuilder = services.AddControllers(options =>
        {
            options.Filters.Add<ExceptionsFilter>();
            options.Filters.Add<IdempotencyFilter>();
            configureMvcOptions?.Invoke(options);
        });

        mvcBuilder.AddNewtonsoftJson(options =>
        {
            options.SerializerSettings.ReferenceLoopHandling = ReferenceLoopHandling.Ignore;
            options.SerializerSettings.ContractResolver = new CamelCasePropertyNamesContractResolver();
            options.SerializerSettings.DateFormatString = "yyyy-MM-dd HH:mm:ss";
            options.SerializerSettings.DateTimeZoneHandling = DateTimeZoneHandling.Local;
            options.SerializerSettings.NullValueHandling = NullValueHandling.Ignore;
            options.SerializerSettings.Converters.Add(new StringEnumConverter());
            configureMvcNewtonsoftJsonOptions?.Invoke(options);
        });
        mvcBuilder.ConfigureApiBehaviorOptions(options =>
        {
            options.InvalidModelStateResponseFactory = _ =>
            {
                var message = new MessageData(false, "the input value not valid", 400);
                return new OkObjectResult(message.Serialize());
            };
            configureApiBehaviorOptions?.Invoke(options);
        });
        configureControllers?.Invoke(mvcBuilder);
        return services;
    }
}